@@ -552,6 +552,9 @@ changed_common(
     {
 	if (wp->w_buffer == curbuf)
 	{
+#ifdef FEAT_FOLDING
+	    linenr_T last = lnume + xtra - 1;  // last line after the change
+#endif
 	    // Mark this window to be redrawn later.
 	    if (wp->w_redr_type < VALID)
 		wp->w_redr_type = VALID;
@@ -561,7 +564,7 @@ changed_common(
 #ifdef FEAT_FOLDING
 	    // Update the folds for this window.  Can't postpone this, because
 	    // a following operator might work on the whole fold: ">>dd".
-	    foldUpdate(wp, lnum, lnume + xtra - 1);
+	    foldUpdate(wp, lnum, last);
 
 	    // The change may cause lines above or below the change to become
 	    // included in a fold.  Set lnum/lnume to the first/last line that
@@ -571,8 +574,8 @@ changed_common(
 	    i = hasFoldingWin(wp, lnum, &lnum, NULL, FALSE, NULL);
 	    if (wp->w_cursor.lnum == lnum)
 		wp->w_cline_folded = i;
-	    i = hasFoldingWin(wp, lnume, NULL, &lnume, FALSE, NULL);
-	    if (wp->w_cursor.lnum == lnume)
+	    i = hasFoldingWin(wp, last, NULL, &last, FALSE, NULL);
+	    if (wp->w_cursor.lnum == last)
 		wp->w_cline_folded = i;
 
 	    // If the changed line is in a range of previously folded lines,
@@ -637,9 +640,13 @@ changed_common(
 	    if (hasAnyFolding(wp))
 		set_topline(wp, wp->w_topline);
 #endif
-	    // Relative numbering may require updating more.
-	    if (wp->w_p_rnu)
-		redraw_win_later(wp, SOME_VALID);
+	    // If lines have been added or removed, relative numbering always
+	    // requires a redraw.
+	    if (wp->w_p_rnu && xtra != 0)
+	    {
+		wp->w_last_cursor_lnum_rnu = 0;
+		redraw_win_later(wp, VALID);
+	    }
 #ifdef FEAT_SYN_HL
 	    // Cursor line highlighting probably need to be updated with
 	    // "VALID" if it's below the change.
