@@ -1146,7 +1146,12 @@ ex_else(exarg_T *eap)
 
     if (eap->cmdidx == CMD_elseif)
     {
-	result = eval_to_bool(eap->arg, &error, eap, skip);
+	// When skipping we ignore most errors, but a missing expression is
+	// wrong, perhaps it should have been "else".
+	if (skip && ends_excmd(*eap->arg))
+	    semsg(_(e_invalid_expression_str), eap->arg);
+	else
+	    result = eval_to_bool(eap->arg, &error, eap, skip);
 
 	// When throwing error exceptions, we want to throw always the first
 	// of several errors in a row.  This is what actually happens when
@@ -1827,6 +1832,16 @@ ex_catch(exarg_T *eap)
 	    cstack->cs_flags[idx] |= CSF_ACTIVE | CSF_CAUGHT;
 	    did_emsg = got_int = did_throw = FALSE;
 	    catch_exception((except_T *)cstack->cs_exception[idx]);
+
+	    if (cstack->cs_idx >= 0
+			       && (cstack->cs_flags[cstack->cs_idx] & CSF_TRY))
+	    {
+		// Variables declared in the previous block can no longer be
+		// used.
+		leave_block(cstack);
+		enter_block(cstack);
+	    }
+
 	    // It's mandatory that the current exception is stored in the cstack
 	    // so that it can be discarded at the next ":catch", ":finally", or
 	    // ":endtry" or when the catch clause is left by a ":continue",
@@ -1930,6 +1945,15 @@ ex_finally(exarg_T *eap)
 	     */
 	    cleanup_conditionals(cstack, CSF_TRY, FALSE);
 
+	    if (cstack->cs_idx >= 0
+			       && (cstack->cs_flags[cstack->cs_idx] & CSF_TRY))
+	    {
+		// Variables declared in the previous block can no longer be
+		// used.
+		leave_block(cstack);
+		enter_block(cstack);
+	    }
+
 	    /*
 	     * Make did_emsg, got_int, did_throw pending.  If set, they overrule
 	     * a pending ":continue", ":break", ":return", or ":finish".  Then
@@ -2283,7 +2307,7 @@ leave_cleanup(cleanup_T *csp)
     {
 	if (pending & CSTP_THROW)
 	    // Cancel the pending exception (includes report).
-	    discard_exception((except_T *)csp->exception, FALSE);
+	    discard_exception(csp->exception, FALSE);
 	else
 	    report_discard_pending(pending, NULL);
 
